import {getCurrentlySelectedList} from "./userlists";
import {getTableAPI} from "./table";

async function checkImportFile() {
    const fileInput = document.querySelector<HTMLInputElement>('#import');

    if (!fileInput) {
        return;
    }

    if (!fileInput.files || !fileInput.files[0]) {
        return;
    }

    const formData = new FormData();
    formData.set('file', fileInput.files[0]);

    const response = await fetch(
        `/api/web/keys/import/prepare`,
        {
            method: 'POST',
            body: formData
        }
    );

    if (!response.ok) {
        throw new Error(response.statusText);
    }

    const jsonData = await response.json();

    const container = document.getElementById('import-info-container');
    if (!container) {
        return
    }

    container.classList.remove('d-none');
    createTableContents(jsonData);
}

function createTableContents(data: {index: string, displayName: string, guessedAttribute: string}[]) {
    const table = document.querySelector<HTMLTableElement>('#import-attribute-table');
    if (!table) {
        return;
    }

    const tbody = table.querySelector<HTMLTableSectionElement>('tbody');
    if (!tbody) {
        return;
    }

    for (let child of tbody.children) {
        child.remove();
    }

    const keyAttributeMeta = document.querySelector<HTMLMetaElement>('meta[name="key-attributes"]')?.content;
    const attributes = JSON.parse(keyAttributeMeta ?? '');

    const select = document.createElement('select');
    select.classList.add('form-select', 'w-100');
    for (let attributeName in attributes) {
        const attributeValue = attributes[attributeName];
        const option = document.createElement('option');
        option.value = attributeValue;
        option.textContent = attributeName;

        select.add(option);
    }

    data.forEach((datum) => {
        const row = tbody.insertRow();
        const indexCell = row.insertCell();
        indexCell.classList.add('index-cell');
        indexCell.innerText = datum.index;

        const displayNameCEll = row.insertCell();
        displayNameCEll.classList.add('name-cell');
        displayNameCEll.innerText = datum.displayName;

        const attributeCell = row.insertCell();
        attributeCell.classList.add('attribute-cell');
        const rowSelect = <HTMLSelectElement>select.cloneNode(true);
        rowSelect.value = datum.guessedAttribute;
        attributeCell.appendChild(rowSelect);
    })
}

async function doImport() {
    const fileInput = document.querySelector<HTMLInputElement>('#import');

    if (!fileInput) {
        return;
    }

    if (!fileInput.files || !fileInput.files[0]) {
        return;
    }

    const formData = new FormData();
    formData.set('file', fileInput.files[0]);

    const tbody = document.querySelector<HTMLTableSectionElement>('#import-attribute-table tbody');
    if (!tbody) {
        return;
    }

    for (let row of tbody.querySelectorAll('tr')) {
        const columnIndex = row.querySelector('.index-cell')?.textContent;
        const attribute = row.querySelector<HTMLSelectElement>('.attribute-cell select')?.value ?? 'none';

        formData.set(`columns[${columnIndex}]`, attribute);
    }

    const listId = getCurrentlySelectedList() ?? 0;
    formData.set('listid', listId.toString());

    const response = await fetch(
        `/api/web/keys/import/perform`,
        {
            method: 'POST',
            body: formData
        }
    );

    if (!response.ok) {
        throw new Error(response.statusText);
    }

    const container = document.getElementById('import-info-container');
    if (!container) {
        return
    }

    container.classList.add('d-none');
    fileInput.value = '';

    getTableAPI().ajax.reload();
}

export function init() {


    const importButton = document.querySelector('.js--send-import');
    importButton?.addEventListener('click', checkImportFile);

    const doImportButton = document.querySelector('.js--do-import');
    doImportButton?.addEventListener('click', doImport);
}