<?php
declare(strict_types=1);

namespace GamesShop\Login;

use Doctrine\ORM\EntityManager;
use GamesShop\Api\DiscordAPI;
use GamesShop\ContainerHandler;
use GamesShop\Entities\Account\User;
use Psr\Http\Message\ServerRequestInterface;

final class DiscordLoginProvider implements LoginProvider
{
    public function __construct(
        private readonly EntityManager $entityManager
    )
    {
    }

    public function getUser(ServerRequestInterface $request): User
    {
        $discordApiHandler = ContainerHandler::get(DiscordAPI::class);
        $result = $discordApiHandler->getUserFromCode($request->getQueryParams()['code'], (string)$request->getUri()->withQuery(''));

        $repo = $this->entityManager->getRepository(User::class);
        $users = $repo->findBy(['loginMethod' => LoginMethod::DISCORD, 'foreignLoginId' => $result['id']]);

        $profilePictureUrl = $discordApiHandler->getAvatarURL($result['id'], $result['avatar'] ?? $result['discriminator'] % 5);
        if (!empty($users)) {
            $user = $users[0];
            $user->setName($result['global_name']);
            $user->setProfilePictureUrl($profilePictureUrl);

            return $user;
        }

        $newUser = new User(
            LoginMethod::DISCORD,
            $result['id'],
            $result['global_name'],
            $profilePictureUrl,
            UserPermission::VIEWER
        );

        return $newUser;
    }
}