<?php
declare(strict_types=1);

namespace GamesShop\Routing\Api\Web;

use GamesShop\Importer\GameImporter;
use GamesShop\Login\LoginHandler;
use GamesShop\Login\UserPermission;
use Laminas\Diactoros\Response\JsonResponse;
use Laminas\Diactoros\UploadedFile;
use League\Route\Http\Exception\BadRequestException;
use League\Route\Http\Exception\ForbiddenException;
use League\Route\Http\Exception\UnauthorizedException;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;

final class ImportKeysPrepareRoute
{
    public function __construct(
        private readonly LoginHandler $loginHandler,
        private readonly GameImporter $importer,
    ) { }

    public function __invoke(ServerRequestInterface $request): ResponseInterface
    {
        if (!$this->loginHandler->isLoggedIn()) {
            throw new UnauthorizedException();
        }

        $user = $this->loginHandler->getCurrentUser();
        if (!$user->getPermission()->hasLevel(UserPermission::PROVIDER)) {
            throw new ForbiddenException();
        }

        /**
         * @var UploadedFile $file
         */
        $file = $request->getUploadedFiles()['file'] ?? null;
        if (!$file === null) {
            throw new BadRequestException();
        }

        $fileName = tempnam(sys_get_temp_dir(), 'ImportKeys');
        $file->moveTo($fileName);
        $results = $this->importer->interpret($fileName);
        unlink($fileName);

        return new JsonResponse($results);
    }
}